package com.example;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;

import com.example.model.Developer;

import java.util.List;

public class DeveloperExample 
{
	private  Session  session = null;

	private  final  String[][]  DEVELOPERS = {
       {"Сергей", "Ветров"      , "Java Developer",  "6", "2400"},
       {"Олег"  , "Мантров"     , "C++ Developer" , "10", "2300"},
       {"Остап" , "Бендер"      , "C# Developer"  ,  "5", "2200"},
       {"Киса"  , "Воробьянинов", "PHP Developer" ,  "4", "2500"}};
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public DeveloperExample()
    {
    	createHibernateSession();
    	
        // Сохранение списка разработчиков
        addDevelopers();
        // Список разработчиков с опытом от 5 лет
        listDevelopers(5);
        // Суммарный заработок разработчиков с опытом от 5 лет
        totalSalary();
        
        session.close();
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Процедура создания сессии
	 * @return org.hibernate.Session
	 */
	private Session createHibernateSession()
	{
		SessionFactory   sf  = null;
		ServiceRegistry  srvc = null;
		try {
			Configuration conf = new Configuration();
			conf.configure("hibernate.cfg.xml");
					
			srvc = new StandardServiceRegistryBuilder().applySettings(
                                              conf.getProperties()).build();
			sf = conf.buildSessionFactory(srvc);
			session = sf.openSession();
			System.out.println("Создание сессии");
		} catch (Throwable e) {
			throw new ExceptionInInitializerError(e);
		}
		return session;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    private Integer addDeveloper(String firstName, String lastName, String specialty, 
    		                     int experience, int salary) 
    {
        Integer developerId = null;

        Developer developer = new Developer();
        developer.setFirstName (firstName );
        developer.setLastName  (lastName  );
        developer.setExperience(experience);
        developer.setSpecialty (specialty );
        developer.setSalary    (salary    );
        developerId = (Integer) session.save(developer);
        
        return developerId;
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    private void addDevelopers()
    {
        Transaction transaction = null;
        transaction = session.beginTransaction();
        
    	for (int i = 0; i < DEVELOPERS.length; i++) {
    		addDeveloper(DEVELOPERS[i][0], DEVELOPERS[i][1], DEVELOPERS[i][2], 
    				     Integer.valueOf(DEVELOPERS[i][3]), 
    				     Integer.valueOf(DEVELOPERS[i][4]));
    	}
        transaction.commit();
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public void listDevelopers(final int min_experience)
    {
        Transaction transaction = session.beginTransaction();
        
        Criteria criteria = session.createCriteria(Developer.class);
        criteria.add(Restrictions.gt("experience", min_experience));
        @SuppressWarnings("unchecked")
		List<Developer> developers = criteria.list();

        for (Developer developer : developers)
            System.out.println(developer);
        
        transaction.commit();
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public void totalSalary() 
    {
        Transaction transaction = session.beginTransaction();
        
        Criteria criteria = session.createCriteria(Developer.class);
        criteria.setProjection(Projections.sum("salary"));

        @SuppressWarnings("unchecked")
		List<Integer> totalSalary = criteria.list();
        System.out.println("Заработок разработчиков : " + totalSalary.get(0));
        
        transaction.commit();
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public static void main(String[] args) 
    {
    	new DeveloperExample();
    	System.exit(0);
    }
}
